from .utils import get_metadata as get_metadata
from typing import List

class Base:
    def __init__(self, options) -> None: ...
    def finish(self) -> None: ...

class Simple(Base):
    def __init__(self, options) -> None: ...
    def __call__(self, meta) -> None: ...

class SingleLine(Base):
    def __init__(self, options) -> None: ...
    def __call__(self, meta) -> None: ...

class CSV(Base):
    def __init__(self, options) -> None: ...
    def __call__(self, meta) -> None: ...

class INI(Base):
    def __init__(self, options) -> None: ...
    def __call__(self, meta) -> None: ...
    def finish(self) -> None: ...

class JSON(Base):
    def __init__(self, options) -> None: ...
    def __call__(self, meta) -> None: ...
    def finish(self) -> None: ...

def main(args: List[str] | None = ...) -> None: ...
